/********************************************************************/
/*                                                                  */
/* Program name: Backend App                                        */
/*                                                                  */
/* Description: This is a Backend application program.              */

/* Written  :   11/08/1999 Humayon Salim                            */
/*                                                                  */
/********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <iostream.h>

//MQ header
#include <imqi.hpp>
// MQ Workflow header
#include "exmwjapc.h"
//#include "fmcjcrun.h"

//********************************************************************
// G L O B A L   D A T A   A N D   F U N C T I O N S
//********************************************************************

#define TRUE  1
#define FALSE 0
#define DELAY 5000

 char   pauseNow[1];
//input structure of the incoming message
typedef struct
{
   char processName[50];
   char processInput[950];
} PROCESS_TRIGGER_DATA;

// the input data structure for the HelloWorld MQWorkflow process
typedef struct
{
  char custNum[6];
} HELLOWORLD_INPUT;

typedef struct
{
  char custNum[7];
} HELLOWORLD_INPUT_STRINGS;

PROCESS_TRIGGER_DATA processTriggerData;

FILE*            outFileStream;
char*            outFile = "BackendApp.out";

//*******************************************************************
//
// RETRIEVE MESSAGE
// This function takes the input queue name that was an input parameter
// to the program and retrieves the incoming message.
// This application makes an ECI call to the appropriate CICS program
// which returns the policy numbers to the bridge application. This
// information is then put onto a reply queue which the function
// retrieves. The policy numbers are then used in subsequent calls to
// CICS programs that return the policy information.
//
//*******************************************************************

void retrieveMessage(char* inputQueueName)
{
  ImqQueueManager         QMgr;             // Queue manager
  ImqQueue                queue;            // Queue
  ImqMessage              msg;              // MQ message
  ImqGetMessageOptions    getMsgOptions;    // Get message options

  char                    msgBuffer[1001];  // buffer for message data

  int                     getContinue;       // flag



  // Connect to the local queue manager
  if ( ! QMgr.connect( ) )
  {
     /* stop if it failed     */
     fprintf( outFileStream, "ImqQueueManager::connect failed with reason code %ld\n",
             QMgr.reasonCode( ) );
     exit(-1);
  }

  // Now we open the input queue
  queue.setName(inputQueueName) ;
  queue.setConnectionReference(QMgr);
  queue.setOpenOptions( MQOO_INPUT_SHARED ) ;
  if ( ! queue.open() )
  {
     fprintf( outFileStream, "\nImqQueue::open ended with reason code %ld\n", queue.reasonCode());
     exit(-1);
  }

  //define where the message data should be put
  msg.useEmptyBuffer(msgBuffer, sizeof(msgBuffer) - 1);

  //set up the get message options
  getMsgOptions.setOptions( MQGMO_WAIT );
  getMsgOptions.setWaitInterval( MQWI_UNLIMITED );

  getContinue = TRUE;

 //  gets(pauseNow);
  printf("\nExample: Activity1(TradeNumber, WaitPeriod in Seconds, Number of Commits to write, Name of Activity)\n");

  while ( getContinue )
  {
     printf("============================================\n");
     printf("\n New Message...\n");
     printf("============================================\n");


     // reset the message and correl ids.
     msg.setMessageId( (MQBYTE*)MQMI_NONE );
     msg.setCorrelationId( (MQBYTE*)MQMI_NONE );

     if ( ! queue.get( msg, getMsgOptions ))
     {
        //the get didn't work for some reason
        fprintf(outFileStream, "\nImqQueue::get ended with reason code %ld\n",
                                                        queue.reasonCode());
        getContinue = FALSE;
     }
     else
     {
        //the get of the reply msg worked
        printf("%s\n", msgBuffer);

        fprintf( outFileStream, "%s\n", msgBuffer);

        // move message buffer into appropriate structure
        memcpy(&processTriggerData, msgBuffer, 1000);

        //finally re-initialize the processTriggerData structure
        memset(&processTriggerData, '\0', 1000);
     }
     fflush(outFileStream);
  }

  if ( ! queue.close( ) )
  {
     fprintf( outFileStream, "\nImqQueue::close ended with reason code %ld\n",
                                                       queue.reasonCode());
     exit(-1);
  }

  // Disconnect from MQM
  if ( ! QMgr.disconnect( ) )
  {
     fprintf( outFileStream, "ImqQueueManager::disconnect failed with reason code %ld\n",
             QMgr.reasonCode( ) ); // report reason, if any
     exit(-1);
  }
}



//********************************************************************
//
// M A I N
//
//********************************************************************

int main(int argc, char * * argv)
{
   char   inputQueueName[100] ;



   //Open the output filename to receive debug messages
   if ((outFileStream = fopen(outFile, "w")) == NULL )
     return -1;

   if (argc == 1)
   {
     // No parameters have been passed at all
     fprintf( outFileStream, "MQIMQWF.EXE Error: No input queue name has been specified\n");
     exit(-1);
   }

   // Obtain the path to the queue configuration from the parameter
   strcpy(inputQueueName, argv[1]);

   printf("InputQueueName is : %s\n", inputQueueName);
   fprintf(outFileStream, "InputQueueName is : %s\n\n", inputQueueName);
   fprintf(outFileStream,"\nExample: Activity1(TradeNumber, WaitPeriod in Seconds, Number of Commits to write, Name of Activity)\n");


   // Get the input message that contans the customer number retrieved from a MQWorkflow process.
   // and any input data to be passed to the process.
   retrieveMessage(inputQueueName);


   fprintf( outFileStream, "Finished processing\n");

   fclose(outFileStream);

   return( 0 );
}

