/********************************************************************/
/*                                                                  */
/* Program name: PutVTrade                                          */
/*                                                                  */
/* Description: Puts a message to the queue.                        */
/*                                                                  */
/* Written  :   11/08/99 Humayon Salim                              */
/*                                                                  */
/********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <iostream.h>

//MQ header
#include <imqi.hpp>

//********************************************************************
// G L O B A L   D A T A   A N D   F U N C T I O N S
//********************************************************************

//backend destination definition
//Defines the name of the backend program that will service the message
//and the queue name to be used to route the information there.

struct policy
{
  char number[6],
       type[5];
};

typedef struct
{
  char retCode[2],
       custNum[6],
       noOfPolicies[2];
  struct policy list[10];
} CUSTPOLS_COMMAREA;

struct policy_type
{
  char policyType[5],
       programName[8],
       RTQ[48],
       noOfPolicies[1],
       policyNumbers[3][6];
} ;


typedef struct
{
   char processName[50];
   char processInput[950];
} PROCESS_TRIGGER_DATA;


PROCESS_TRIGGER_DATA processTriggerData;

CUSTPOLS_COMMAREA custpolsCommarea;

FILE*            outFileStream;
char*            outFile = "PutVTrade.out";

char MessageId[24];


//*******************************************************************
//
// SendMessageToQueue
// This function takes the data from the end-user and is put on an
// MQSeries Queue.
//
//*******************************************************************

void SendMessageToQueue(ImqQueueManager* QMgr)
{
  ImqQueue queue;                        // Queue
  ImqMessage msg, replyMsg;              // MQ message
  int i, wait_time, noOfPoliciesInt;
  char messageData[128];                  // message data
  char replyMsgBuffer[129];               // buffer for reply message data
  char commAreaData[121], noOfPoliciesString[3], policyType[6];
  ImqGetMessageOptions getMsgOptions;
  char* stringData;

  //set the format of the message
  msg.setFormat( MQFMT_STRING );
  msg.setMessageType( MQMT_REQUEST );

  //set the name of the backend queue
  queue.setName("LOG_QUEUE");

  //set the name of the reply queue in the message
  msg.setReplyToQueueName("LOG_QUEUE");

  // set up the message to be sent
  // add the name of the Backend program to invoke.
  stringData = "Activity(100123, 2, 1, CalculateNewDepositPosition )";


  strcpy(messageData, stringData);


  strcat(messageData, (char*)&custpolsCommarea);

  // Display start of MQSeries log...
  fprintf( outFileStream, "\n================Screen1(front-end) - Start of MQSeries Log ===================\n");


  //DEBUG
  fprintf(outFileStream,"\n1) The Message has been sent to queue LOG_QUEUE.\n");
  fprintf(outFileStream,"\n2) The Message content is:   %s\n", messageData);

  //link the queue to the queue manager
  queue.setConnectionReference(QMgr);

  //set the open options to be used when opening the queue
  queue.setOpenOptions( MQOO_OUTPUT /* open queue for output        */
#if defined ( _IMQ_VER ) && ( _IMQ_VER >= 200 )
      + MQOO_FAIL_IF_QUIESCING
#endif
                       );     /* but not if MQM stopping      */

  if ( ! queue.open( ) )
  {
     fprintf( outFileStream, "ImqQueue::open ended with reason code %ld\n",
              queue.reasonCode( ) );
     exit( (int)queue.reasonCode( ) );
  }

  //tell the message where to get message data from
  msg.useFullBuffer(messageData, 128);

  // Put the message on the queue
  if ( ! queue.put( msg ) )
  {
     fprintf( outFileStream, "ImqQueue::put ended with reason code %ld\n",
             queue.reasonCode( ) );
     exit( (int)queue.reasonCode( ) );
  }

  //Close the queue
  if ( ! queue.close( ) )
  {
     /* report reason, if any     */
     fprintf( outFileStream, "ImqQueue::close ended with reason code %ld\n",
             queue.reasonCode( ) );
     exit( (int)queue.reasonCode( ) ) ;
  }

}

//********************************************************************
//
// M A I N
//
//********************************************************************

int main(int argc, char * * argv)
{
   FILE* 		configFileStream;
   char* 		customerNumber;
   char 		configFile[100];
   ImqQueueManager 	QMgr;

   char*  pSessionId;   // session id required for MQ Workflow API calls


   //Open the output filename to receive debug messages
   if ((outFileStream = fopen(outFile, "w")) == NULL )
     return -1;



   //we now have the customer number. Now get all of the details from the backend application.
   // Connect to the local queue manager
   if ( ! QMgr.connect( ) )
   {
      /* stop if it failed     */
      fprintf( outFileStream, "ImqQueueManager::connect failed with reason code %ld\n",
              QMgr.reasonCode( ) );
      exit(-1);
   }


   // Put an MQ Message onto the appropriate queue to trigger a Backend app that will
   //return all of the backend program details.
   SendMessageToQueue(&QMgr);

   // Disconnect from MQM
   if ( ! QMgr.disconnect( ) )
   {
      fprintf( outFileStream, "ImqQueueManager::disconnect failed with reason code %ld\n",
              QMgr.reasonCode( ) ); // report reason, if any
      exit(-1);
   }

   return( 0 );
}

